struct PersistenceDiagramRowIterator{D}
    diagram::D
end

function Base.iterate(it::PersistenceDiagramRowIterator, i=1)
    if i > length(it.diagram)
        return nothing
    else
        int = it.diagram[i]
        dim = get(it.diagram.meta, :dim, missing)
        threshold = get(it.diagram.meta, :threshold, missing)
        return (birth=int.birth, death=int.death, dim=dim, threshold=threshold), i + 1
    end
end

Base.IteratorSize(::PersistenceDiagramRowIterator) = Base.HasLength()
Base.length(it::PersistenceDiagramRowIterator) = length(it.diagram)

Tables.istable(::Type{<:PersistenceDiagram}) = true
Tables.rowaccess(::Type{<:PersistenceDiagram}) = true
function Tables.rows(diagram::PersistenceDiagram)
    PersistenceDiagramRowIterator(diagram)
end
function Tables.schema(it::PersistenceDiagramRowIterator)
    diagram = it.diagram
    D = hasproperty(diagram, :dim) ? Int : Missing
    T = hasproperty(diagram, :threshold) ? Float64 : Missing
    return Tables.Schema(
        (:birth, :death, :dim, :threshold),
        (Float64, Float64, D, T),
    )
end

Tables.materializer(::PersistenceDiagram) = PersistenceDiagram

"""
    PersistenceDiagrams.table(::AbstractVector{<:PersistenceDiagram})

Wrap a vector of `PersistenceDiagram`s in a `PersistenceDiagramTable`, which satisfies the
Tables.jl interface.
"""
table(ds::AbstractVector{<:PersistenceDiagram}) = PersistenceDiagramTable(ds)
table(d::PersistenceDiagram) = d

struct PersistenceDiagramTable{V<:AbstractVector{<:PersistenceDiagram}}
    diagrams::V
end

function Base.iterate(it::PersistenceDiagramTable, st=(1, 1))
    i, j = st
    if j > length(it.diagrams)
        return nothing
    end
    if i > length(it.diagrams[j])
        i = 1; j += 1
    end
    if j > length(it.diagrams)
        return nothing
    end
    diagram = it.diagrams[j]
    int = diagram[i]
    dim = get(diagram.meta, :dim, missing)
    threshold = get(diagram.meta, :threshold, missing)
    return (birth=int.birth, death=int.death, dim=dim, threshold=threshold), (i + 1, j)
end

Base.IteratorSize(::PersistenceDiagramTable) = Base.HasLength()
Base.length(table::PersistenceDiagramTable) = sum(length.(table.diagrams))

Tables.isrowtable(::Type{<:PersistenceDiagramTable}) = true

function Tables.schema(table::PersistenceDiagramTable)
    diagrams = table.diagrams
    D = all(d -> hasproperty(d, :dim), diagrams) ? Int : Union{Int, Missing}
    T = all(d -> hasproperty(d, :threshold), diagrams) ? Float64 : Union{Float64, Missing}
    return Tables.Schema(
        (:birth, :death, :dim, :threshold),
        (Float64, Float64, D, T),
    )
end
